import * as tslib_1 from "tslib";
import PropTypes from 'prop-types';
import deepmerge from 'deepmerge'; // < 1kb payload overhead when lodash/merge is > 3kb.
import warning from 'warning';
//material-ui code, used for both web and native
import createBreakpoints from 'material-ui/styles/createBreakpoints';
import createPalette from 'material-ui/styles/createPalette';
import createMixins from 'material-ui/styles/createMixins';
import transitions from 'material-ui/styles/transitions';
import zIndex from 'material-ui/styles/zIndex';
import spacing from 'material-ui/styles/spacing';
//platform specific code
import { createTypography, shadows, toPlatformRuleSet, toPlatformSheet } from 'muix-styles';
export const classesToPlatformSheet = (theme, classes) => {
    const sheetx = typeof classes === 'function' ? classes(theme) : classes;
    return toPlatformSheet(sheetx); //Muix.PartialSheetX<Muix.Shape>
};
//create platform specific sheet from cross platform sheet creator
export const sheetCreator = (sheetXCreator) => ((theme) => toPlatformSheet(sheetXCreator(theme)));
//create platform specific ruleset from cross platform ruleset
export const toPlatformRuleSetX = (style, isNative) => {
    if (!style)
        return null;
    const { $web, $native, $overrides, $childOverrides } = style, rest = tslib_1.__rest(style, ["$web", "$native", "$overrides", "$childOverrides"]);
    const res = Object.assign({}, rest, (isNative ? $native : $web), { $overrides: toPlatformSheetX($overrides, isNative), $childOverrides: getOverridesX(null, $childOverrides) });
    if (!res.$overrides)
        delete res.$overrides;
    if (!res.$childOverrides)
        delete res.$childOverrides;
    return res;
};
export const clearSystemProps = obj => {
    if (!obj)
        return obj;
    delete obj.$overrides;
    delete obj.$childOverrides;
    delete obj.$name;
    delete obj.$web;
    delete obj.$native;
    return obj;
};
//create platform specific sheet from cross platform sheet
export const toPlatformSheetX = (sheet, isNative) => {
    if (!sheet)
        return null;
    const res = {};
    for (const p in sheet)
        res[p] = toPlatformRuleSetX(sheet[p], isNative);
    return res;
};
//create platform specific Overrides from cross platform Overrides
const getOverridesX = (theme, source) => {
    if (!source)
        return null;
    if (typeof source === 'function')
        source = source(theme);
    const result = {};
    for (const p in source)
        result[p] = toPlatformSheet(source[p]);
    return result;
};
////convert cross platform typography optionsOrCreator to platform specific optionsOrCreator
//const getTypographyOptionOrCreatorX = (optionsOrCreatorX: Muix.TypographyOptionsOrCreatorX) => {
//  const getOptionX = (optionsX: Muix.TypographyOptionsX) => {
//    let res: Muix.TypographyOptionsNew = {}
//    if (optionsX) {
//      const { fontFamily, fontSize, htmlFontSize, ...rulesX } = optionsX
//      const rules: PartialRecord<Muix.typoStyle, ReactN.TextStyle> = {}
//      for (const p in rulesX) rules[p] = toPlatformRuleSet(rulesX[p]) //toPlatformRuleSet is platform specific
//      res = {
//        fontFamily, fontSize, htmlFontSize,
//        ...rules
//      }
//    }
//    return res
//  }
//  let res: Muix.TypographyOptionsOrCreator = {}
//  if (optionsOrCreatorX) {
//    if (typeof optionsOrCreatorX == 'function') res = palette => getOptionX(optionsOrCreatorX(palette))
//    else res = getOptionX(optionsOrCreatorX)
//  }
//  return res
//}
export const MuiThemeContextTypes = { theme: PropTypes.any };
export const MuiOverridesContextTypes = { childOverrides: PropTypes.any };
let defaultTheme;
export const getDefaultTheme = () => defaultTheme || (defaultTheme = createMuiTheme());
//create theme from cross platform ThemeOptions
//resulting theme is compatible with material-ui
function createMuiTheme(options = {}) {
    const { palette: paletteInput = {}, breakpoints: breakpointsInput = {}, mixins: mixinsInput = {}, 
    //use cross platform typography options instead
    typographyX, typography: ignored1, //ignored, use cross platform typographyNew instead
    //use cross platform shadows options instead
    shadowsX: shadowsNewInputX, shadows: ignored2, //ignored, use cross platform shadowsNew instead
    //use cross platform overrides options instead
    overridesX, overrides: ignored3 } = options, //ignored , use cross platform overridesNew instead
    other = tslib_1.__rest(options
    //convert cross platform shadows to platform specific shadows
    , ["palette", "breakpoints", "mixins", "typographyX", "typography", "shadowsX", "shadows", "overridesX", "overrides"]) //contains direction, transitions, spacing, zIndex
    ;
    //convert cross platform shadows to platform specific shadows
    const shadowsNewInput = shadowsNewInputX && shadowsNewInputX.map(rsx => toPlatformRuleSet(rsx));
    const palette = createPalette(paletteInput);
    const breakpoints = createBreakpoints(breakpointsInput);
    //const typographyOptionOrCreator = getTypographyOptionOrCreatorX(typographyNew)
    const muiTheme = Object.assign({ direction: 'ltr', //the same value for web and native
        palette }, createTypography(palette, typographyX), { mixins: createMixins(breakpoints, spacing, mixinsInput), //the same value for web and native
        breakpoints, shadows: (shadowsNewInput || shadows).map(rsx => toPlatformRuleSetX(rsx, false).boxShadow), shadowsNew: shadowsNewInput || shadows }, (deepmerge({ transitions, spacing, zIndex, }, other, { clone: false })));
    muiTheme.overrides = getOverridesX(muiTheme, overridesX); //different value for web and native
    warning(muiTheme.shadows.length === 25 && muiTheme.shadowsNew.length === 25, 'MUIX: the shadows array provided to createMuiTheme should support 25 elevations.');
    return muiTheme;
}
export default createMuiTheme;
