import * as tslib_1 from "tslib";
import React from 'react';
import hoistNonReactStatics from 'hoist-non-react-statics';
import { MuiThemeContextTypes, MuiOverridesContextTypes, getDefaultTheme, classesToPlatformSheet } from '../common/index';
import { toPlatformRuleSet, clearSystemProps } from 'muix-styles';
const withStyles = (sheetOrCreator, options) => (Component) => {
    class Styled extends React.PureComponent {
        constructor(props, context) {
            super(props, context);
            this.usedChildOverrides = {};
            const { flip, name } = options;
            const _a = props, { classes: _classes, style, web, native, onClick } = _a, other = tslib_1.__rest(_a, ["classes", "style", "web", "native", "onClick"]);
            const onPressNative = native && native.onPress;
            const onClickWeb = web && web.onClick;
            let theme = context.theme || getDefaultTheme();
            //caching aplyThemeToSheet result in actual theme (in its .$sheetCache prop)
            if (!theme.$sheetCache)
                theme.$sheetCache = [];
            let cacheItem = theme.$sheetCache.find(it => it.sheetOrCreator === sheetOrCreator);
            if (!cacheItem)
                theme.$sheetCache.push(cacheItem = { sheetOrCreator, fromTheme: aplyThemeToSheet(sheetOrCreator, theme, name) });
            //console.log('1', toPlatformSheet({ common, native: classesNative, web: classesWeb } as Mui.PartialSheetX<R>))
            //console.log(name, 'context.childOverrides', context.childOverrides)
            const fromParentContext = context.childOverrides && context.childOverrides[options.name];
            //console.log('fromParentContext: ',fromParentContext)
            this.codeClasses = fromParentContext ? deepMerges(false, {}, cacheItem.fromTheme, fromParentContext) : cacheItem.fromTheme; // modify static sheet 
            //console.log('cacheItem.fromTheme: ',cacheItem.fromTheme)
            for (const p in this.codeClasses)
                this.codeClasses[p].$name = p; // assign name to ruleSets. $name is used in getStyleWithSideEffect to recognize used rulesets
            // Could be called in <Component> render method to compute component styles. Side effects:
            // - use sheet..$overrides to modify self sheet
            // - sheet..$childOverrides to modify children sheet (passed to children via context.childOverrides) 
            const classesProp = classesToPlatformSheet(theme, _classes);
            //if (classesProp) console.log('### classesProp', classesProp)
            const usedOverrides = {};
            const getStyleWithSideEffect = (...rulesets /*all used rulesets*/) => {
                //console.log('getStyleWithSideEffect', rulesets)
                rulesets.forEach(ruleset => {
                    if (!ruleset)
                        return;
                    mergeOverride(usedOverrides, ruleset.$overrides);
                    mergeOverride(this.usedChildOverrides, ruleset.$childOverrides); //modify react context for 
                });
                //console.log('this.usedChildOverrides', this.usedChildOverrides)
                const rulesetResult = {};
                rulesets.forEach(ruleset => {
                    if (!ruleset)
                        return;
                    //if (classesProp && ruleset.$name=='root') console.log('### BEFORE', rulesetResult)
                    deepMerges(true, rulesetResult, /*ruleset, used in Component render*/ ruleset, /*modify it with used $overrides*/ usedOverrides[ruleset.$name], /*force using classes component property, its rulesets cannot be overrided */ classesProp && classesProp[ruleset.$name]);
                    //if (classesProp && ruleset.$name == 'root') console.log('### AFTER', rulesetResult)
                });
                return rulesetResult;
            };
            this.newProps = Object.assign({}, other, (window.isWeb ? web : native), { theme, style: clearSystemProps(toPlatformRuleSet(style)), flip: typeof flip === 'boolean' ? flip : theme.direction === 'rtl', getStyleWithSideEffect });
            if (window.isWeb)
                this.newProps.onClick = onClickWeb || onClick;
            else
                this.newProps.onPress = onPressNative || onClick;
        }
        getChildContext() { return { childOverrides: this.usedChildOverrides /*usedChildOverrides is modified during Component render (where getStyleWithSideEffect is called)*/ }; }
        render() {
            this.newProps.classes = this.codeClasses;
            return <Component {...this.newProps}/>;
        }
    }
    Styled.contextTypes = Object.assign({}, MuiThemeContextTypes, MuiOverridesContextTypes);
    Styled.childContextTypes = MuiOverridesContextTypes;
    Styled.options = options;
    hoistNonReactStatics(Styled, Component);
    //return Styled: error when compiling with "declaration": true in tsconfig.json
    const styled = Styled;
    return styled;
};
export default withStyles;
//apply theme to sheet AND merge it with theme.overrides
const aplyThemeToSheet = (sheetOrCreator, theme, name) => {
    const overrides = (theme.overrides && name && theme.overrides[name]);
    //console.log('###BEFORE', overrides)
    const styles = (typeof sheetOrCreator === 'function' ? sheetOrCreator(theme) : sheetOrCreator);
    const res = overrides ? deepMerges(false, {}, styles, overrides) : styles; //deepMerge only when needed
    //console.log('###AFTER', res.root)
    return res;
};
// merge named values
const mergeOverride = (result, patches) => {
    if (!patches)
        return;
    for (const p in patches) {
        const patch = patches[p];
        if (!patch)
            continue;
        if (!result[p])
            result[p] = {};
        deepMerge(result[p], patch, false);
    }
};
//simple deep merge
export const deepMerge = (target, source, skipSystem = false) => {
    if (!source)
        return target;
    if (isObject(target) && isObject(source))
        for (const key in source) {
            if (skipSystem && key[0] === '$')
                continue; //skip $override, $overrides and $name props
            if (isObject(source[key])) {
                if (!target[key])
                    target[key] = {};
                deepMerge(target[key], source[key], skipSystem);
            }
            else
                target[key] = source[key];
        }
    else
        throw 'deepMerge: cannot merge object and non object';
    return target;
};
const deepMerges = (skipSystem, target, ...sources) => {
    sources.forEach(source => deepMerge(target, source, skipSystem));
    return target;
};
const isObject = item => item && typeof item === 'object' && !Array.isArray(item);
